import numpy as np
from keras.applications.inception_v3 import preprocess_input
from pathlib import Path
import math
import uuid
from werkzeug.utils import secure_filename
from flask import Flask, request, jsonify
import mimetypes
import model.imagegenerator as im
import cv2
import tensorflow_hub as hub
import os
import model.load_model as lm
import predictor as pd
global hub
global model, graph

# global variables
feature_extractor_url = 'https://tfhub.dev/google/imagenet/inception_v3/feature_vector/3'
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '3'
script_location = Path(__file__).absolute().parent

app = Flask(__name__, static_folder='upload')
app.config.from_mapping(
    SECRET_KEY='dev',
    DATABASE=os.path.join(app.instance_path, 'dog-breed.sqlite'),
)
app.config['UPLOAD_FOLDER'] = script_location / 'upload'


# load model
model, graph = lm.ModelInitializer()


def get_file_type(filename):
    mimetypes.init()
    mimestart = mimetypes.guess_type(filename)[0]
    return mimestart


def save_image(frame):
    filename = secure_filename('{}.{}'.format(uuid.uuid4().hex, 'jpg'))
    filepath = os.path.join(app.config['UPLOAD_FOLDER'], filename)
    cv2.imwrite(filepath, frame)
    return filename


def predict_from_video(pred_target):
    cap = cv2.VideoCapture(pred_target)

    # 5 == parameter select frame rate of video source
    frameRate = cap.get(5)

    # predict dog in each frame
    results = []
    i = 1
    while(cap.isOpened()):
        frameId = cap.get(1)  # current frame number
        ret, frame = cap.read()
        if (ret != True):
            break
        if (frameId % math.floor(frameRate) == 0):
            i += i
            img_tensor = im.InputImageGeneratorVideo(frame)
            prediction = pd.Predict(img_tensor, model, graph)
            prediction['frame'] = frame
            results.append(prediction)
    cap.release()

    # get breed which has maximum probability of the list of results
    predicted_breed = max(results, key=lambda x: x['probability'])

    # add saved image name
    predicted_breed['image'] = save_image(predicted_breed['frame'])

    # remove frame from the dict
    del predicted_breed['frame']
    return predicted_breed


def predict_from_image(pred_target):
    img_tensor = im.InputImageGeneratorImage(pred_target)
    prediction = pd.Predict(img_tensor, model, graph)
    return prediction


# index endpoint, can use to check server status
@app.route('/', methods=['GET', 'POST'])
def hello():
    return ("Server is running!")


@app.route('/predict', methods=['POST'])
def get_breed_from_image():
    # check if the post request has the file part
    if 'file' not in request.files:
        return jsonify(dict({'message': 'No file attached to the request.'}))

    # uploaded file
    file = request.files['file']

    # if user does not select file, browser also submit an empty part without filename
    if file.filename == '':
        return jsonify(dict({'message': 'No selected file.'}))

    # check file type
    mime = get_file_type(file.filename)
    filetype, extension = mime.split('/')
    if str(filetype) not in ('image', 'video'):
        return jsonify(dict({'message': 'File type is not allowed'}))

    # create new filename
    filename = secure_filename('{}.{}'.format(uuid.uuid4().hex, extension))

    # save video in tempory folder
    filepath = os.path.join(app.config['UPLOAD_FOLDER'], filename)
    file.save(filepath)

    # predict base on the type of file
    result = {}
    if filetype == 'video':
        result = predict_from_video(filepath)

        # delete video file since we don't need it anymore
        os.remove(filepath)
    else:
        result = predict_from_image(filepath)

        # add filename of image to result
        result['image'] = filename

    return jsonify(result)


if __name__ == "__main__":
    app.run(debug=True)
